@@ -22,6 +22,7 @@
 #include <sys/time.h>
 #include <sys/sysinfo.h>
 #include <sys/select.h>
+#include <sys/stat.h>
 #include <dirent.h>
 #include <errno.h>
 #include <stdio.h>
@@ -337,8 +338,23 @@ int get_fsid(const char *path, u8 *fsid, int silent)
 {
 	int ret;
 	int fd;
+	int flags = O_RDONLY;
+	struct stat st;
 
-	fd = open(path, O_RDONLY);
+	ret = stat(path, &st);
+	if (ret < 0) {
+		if (!silent)
+			error("failed to stat %s: %m", path);
+		return -errno;
+	}
+	/*
+	 * Open in non-blocking mode in case that path is a fifo or a special
+	 * character device where opening gets stuck (but is interruptible).
+	 */
+	if ((st.st_mode & S_IFMT) == S_IFCHR || (st.st_mode & S_IFMT) == S_IFIFO)
+		flags |= O_NONBLOCK;
+
+	fd = open(path, flags);
 	if (fd < 0) {
 		if (!silent)
 			error("failed to open %s: %m", path);
@@ -944,6 +960,7 @@ void btrfs_config_init(void)
 {
 	bconf.output_format = CMD_FORMAT_TEXT;
 	bconf.verbose = BTRFS_BCONF_UNSET;
+	INIT_LIST_HEAD(&bconf.params);
 }
 
 void bconf_be_verbose(void)
@@ -959,6 +976,57 @@ void bconf_be_quiet(void)
 	bconf.verbose = BTRFS_BCONF_QUIET;
 }
 
+void bconf_add_param(const char *key, const char *value)
+{
+	struct config_param *param;
+
+	param = calloc(1, sizeof(*param));
+	if (!param)
+		return;
+	param->key = strdup(key);
+	if (value)
+		param->value = strdup(value);
+	list_add(&param->list, &bconf.params);
+}
+
+const char *bconf_param_value(const char *key)
+{
+	struct config_param *param;
+
+	list_for_each_entry(param, &bconf.params, list) {
+		if (strcmp(key, param->key) == 0)
+			return param->value;
+	}
+	return NULL;
+}
+
+void bconf_save_param(const char *str)
+{
+	char *tmp;
+
+	tmp = strchr(str, '=');
+	if (!tmp) {
+		bconf_add_param(str, NULL);
+		printf("Global param: %s\n", str);
+	} else {
+		*tmp = 0;
+		bconf_add_param(str, tmp + 1);
+		printf("Global param: %s=%s\n", str, tmp + 1);
+		*tmp = '=';
+	}
+}
+
+void bconf_set_dry_run(void)
+{
+	pr_verbose(LOG_INFO, "Dry-run requested\n");
+	bconf.dry_run = 1;
+}
+
+bool bconf_is_dry_run(void)
+{
+	return bconf.dry_run == 1;
+}
+
 /* Returns total size of main memory in bytes, -1UL if error. */
 unsigned long total_memory(void)
 {
