@@ -18,9 +18,8 @@
  */
 
 #include <errno.h>
-#include <stdarg.h>
 #include <stdlib.h>
-#include <string.h>
+#include <stdint.h>
 
 #include "btrfsutil_internal.h"
 
